# Release Notes: 0.8

## Composable directives

Through composable directives, we can modify or augment the behavior of directives. This is useful since, often times, a directive cannot be applied on the field, because it has an input which is different than the field's output.

For instance, `User.capabilities` returns `[String]` (an array of strings), and directive `@upperCase` receives `String`. Hence, executing the following query returns an error due to the type mismatch:

```graphql
{
  users {
    capabilities @upperCase
  }
}
```

A nested directive can fill the gap between these two types, transforming the output from the field into the required input for the directive.

For instance, directive `@forEach` can iterate over an array of elements, and apply its nested directive on each of them. The query from above can be satisfied like this (where param `nestedUnder` indicates the relative position of the parent directive):

```graphql
{
  users {
    capabilities
      @forEach
      @upperCase(nestedUnder: -1)
  }
}
```

Composable directives keep the schema clean and elegant, allowing the minimum set of elements to satisfy a great amount of functionality.

This functionality is an opt-in, because it is non-standard (it is not part of the GraphQL spec).
